//
//  ATKCameraController.h
//  ExampleApplication
//
//  Created by Nick Kitchener on 24/01/2013.
//  Copyright (c) 2013 ATIK. All rights reserved.
//
//  A threaded camera controller

#import <Foundation/Foundation.h>
#import <ATIKOSXDrivers/ATIKOSXDrivers.h>
#import "ATKLiveUpdateDelegate.h"
#import "ATKControllerManagement.h"
#import "ATKImageSub.h"

@interface ATKCameraController : NSObject <ATKControllerManagement, StateObserver> {
    
    id<Imager100>   _imager;
    dispatch_queue_t postProcessingQueue;   // PARALLEL for images post processing, writing images to disk
    dispatch_queue_t commandQueue;          // SERIAL for all camera operations
    float           _targetTemperature;
    ATKImageSub      *_liveviewImage;
    NSObject<StateObserver> *_stateObserver;
    
    ImageFrame      *imageA;
    ImageFrame      *imageB;
    ATKImageSub *subA;
    ATKImageSub *subB;
    
    uint32_t        global_gn; // sequence
    
    float           oldSetPointTemperature;
    float           setPointTemperature;
    float           currentTemperature;
    BOOL            pollSet;
    
    NSTimer         *temperaturePollTimer;
    
    BOOL            abortPreview;
    BOOL            abortCaptureSequence;
    BOOL            shutdownQueue;
    
#if __MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_8
    BOOL isPreviewModeActive; // this is the camera preview mode NOT the example app
    NSString *identity;
    NSImage    *icon;
    NSString   *name;
    NSNumber *duration;
    NSNumber *previewMode;
    NSNumber *binX;
    NSNumber *binY;
    NSObject<ATKLiveUpdateDelegate> *liveView;
    NSNumber *targetTemperature;
    NSNumber *temperature;
    NSNumber    *enableSubsample;
    NSNumber    *enablePrecharge;
    NSNumber    *enableTitanAutoBlackLevel;
    NSNumber    *prechargeValue;
    NSString    *currentState;
#endif
    
}

@property (readwrite, assign, nonatomic) BOOL isPreviewModeActive; // this is the camera preview mode NOT the example app

@property(readwrite, assign, nonatomic) NSString *identity;
@property (readwrite, assign, nonatomic) NSImage    *icon;

@property (readonly, assign, nonatomic) NSString   *name;
@property (readwrite, retain, nonatomic) NSNumber *duration;
@property (readwrite, assign, nonatomic) NSNumber *previewMode;
@property (readwrite, assign, nonatomic) NSNumber *binX;
@property (readwrite, assign, nonatomic) NSNumber *binY;

@property (readwrite, assign, nonatomic) NSObject<ATKLiveUpdateDelegate> *liveView;
@property (readwrite, assign, atomic) NSNumber *targetTemperature;
@property (readwrite, assign, atomic) NSNumber *temperature;

@property(readwrite, assign, nonatomic) NSNumber    *enableSubsample;
@property(readwrite, assign, nonatomic) NSNumber    *enablePrecharge;
@property(readwrite, assign, nonatomic) NSNumber    *enableTitanAutoBlackLevel;
@property(readwrite, assign, nonatomic) NSNumber    *prechargeValue;

@property(readwrite, assign, nonatomic) NSString    *currentState;

-(void)setStateObserver:(NSObject<StateObserver>*)observer;

-(id)initWithImager:(id<Imager100>)providedImager;

-(void)setState:(NSString*)newState;

-(NSImage *)iconForImagerType:(id<Imager100>)imager;

-(void) testExposure;
-(void) startPreviewMode;
-(void) startCapture:(uint32_t)count ofDuration:(float)duration toURL:(NSURL*)targetURL;
-(void) startWarmupCycle;

-(void)abort;

-(void)pollTemperatureUpdate:(NSTimer*)timer;

-(void)shutdown;

-(void)guide:(long)direction;

@end

