//
//  ATKCameraController.m
//  ExampleApplication
//
//  Created by Nick Kitchener on 24/01/2013.
//  Copyright (c) 2013 atik. All rights reserved.
//

#import "ATKCameraController.h"

extern "C++" {
#include <ATIKOSXDrivers/ArtemisDeviceDisconnectedException.h>
};

#import <ATIKOSXDrivers/ATIKExtensions.h>

@interface ATKCameraController ()
@end

@implementation ATKCameraController

#if __MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_8
@synthesize isPreviewModeActive=_isPreviewModeActive; // this is the camera preview mode NOT the example app
@synthesize identity=_identity;
@synthesize icon=_icon;
@synthesize name=_name;
@synthesize duration=_duration;
@synthesize previewMode=_previewMode;
@synthesize binX=_binX;
@synthesize binY=_binY;
@synthesize liveView=_liveView;
@synthesize targetTemperature=_targetTemperature;
@synthesize temperature=_temperature;
@synthesize enableSubsample=_enableSubsample;
@synthesize enablePrecharge=_enablePrecharge;
@synthesize enableTitanAutoBlackLevel=_enableTitanAutoBlackLevel;
@synthesize prechargeValue=_prechargeValue;
@synthesize currentState=_currentState;
#endif

-(id)initWithImager:(id)providedImager {
    self = [super init];
    if( self ) {
        shutdownQueue=NO;
        _imager = providedImager;
        _icon = [self iconForImagerType:(id<Imager100>)providedImager];
        _binX = [NSNumber numberWithInt:1];
        _binY = [NSNumber numberWithInt:1];
        _previewMode = [NSNumber numberWithBool:NO];
        _duration = [NSNumber numberWithFloat:1.0];
        _targetTemperature = 20.0;
        _currentState = @"pending camera";
        
        _enablePrecharge = NO;
        _enableSubsample = NO;
        _enableTitanAutoBlackLevel = NO;
        
        imageA = nil;
        imageB = nil;
        
        currentTemperature = 999.0;
        oldSetPointTemperature=setPointTemperature = 999.0;
        pollSet=NO;
        
        global_gn = 0;
        _liveviewImage=nil;
                
        NSString *postProcessingQueueName = [NSString stringWithFormat:@"com.atik.exampleapp.%@.postprocess",[providedImager uniqueIdentity]];
        postProcessingQueue = dispatch_queue_create([postProcessingQueueName UTF8String], DISPATCH_QUEUE_CONCURRENT);
        
        NSString *commandQueueName = [NSString stringWithFormat:@"com.atik.exampleapp.%@.command",[providedImager uniqueIdentity]];
        commandQueue = dispatch_queue_create([commandQueueName UTF8String], DISPATCH_QUEUE_SERIAL);
                
        NSLog(@"Adding timer..");
        temperaturePollTimer = [NSTimer timerWithTimeInterval:5.0 target:self selector:@selector(pollTemperatureUpdate:) userInfo:nil repeats:YES];
        [[NSRunLoop mainRunLoop] addTimer:temperaturePollTimer forMode:NSDefaultRunLoopMode];
     }
    return self;
}

-(void)dealloc {
#if ! __has_feature(objc_arc)
    [super dealloc];
#endif
}

-(void)shutdown {
    // suspend queues - 
    dispatch_suspend(commandQueue);
    dispatch_suspend(postProcessingQueue);
    shutdownQueue=YES;
    dispatch_resume(commandQueue);
    dispatch_resume(postProcessingQueue);
    
    [temperaturePollTimer invalidate];
    temperaturePollTimer = nil;
    [_imager abortCapture];
    }

-(void)setState:(NSString*)newState {
    [_liveView performSelectorOnMainThread:@selector(stateState:) withObject:newState waitUntilDone:YES];
}

-(void)setStateObserver:(NSObject<StateObserver>*)observer {
    _stateObserver = observer;
    [_imager setStateObserver:observer];
}

-(NSImage *)iconForImagerType:(id<Imager100>)imager {
    NSString *thumbnameName=@"unknown";
    return [NSImage imageNamed:thumbnameName];
}

-(void) startPreviewMode {
    abortPreview = NO;
    dispatch_async(commandQueue, ^{
        try {
        [self previewLoop];
        } catch(ArtemisDeviceDisconnectedException &e) {
        } catch(...) {
        }
    });
}

-(void) stopPreviewMode {
    abortPreview = YES;
}

-(void) startCapture:(uint32_t)count ofDuration:(float)duration toURL:(NSURL*)targetURL {
    [self stopPreviewMode]; // stop preview the next set of captures will then queue behind
    dispatch_async(commandQueue, ^{
        try{
            [self capture:count ofDuration:duration toURL:targetURL];
        }  catch(ArtemisDeviceDisconnectedException &e) {
        } catch(...) {
        }
    });
}

// take a 1 second exposure from the camera.
-(void)testExposure {
    dispatch_async(commandQueue, ^{
        try {
            [self takeExposure:1.0f updateGUI:YES saveToFile:nil];
        } catch(ArtemisDeviceDisconnectedException &e) {
        } catch(...) {
        }
    });
}


-(void)previewLoop {
    NSLog(@"Preview mode started");
    //while([self isPreviewModeActive]) {
    for(int i=0;(i<10000) && !abortPreview && !shutdownQueue;i++) {
        [self takeExposure:[[self duration]floatValue] updateGUI:YES saveToFile:nil];
    }
    if( abortPreview ) {
        NSLog(@"Preview mode aborted.");
    }
    abortPreview = NO;
    NSLog(@"Preview mode ended");
}

// creates a date + sequence name for the image in the root URL location.
-(NSURL*)createURL:(NSURL*)rootURL {
    NSString *d = [NSDate date];
    NSString *filename;
    @synchronized(self) {
        filename = [NSString stringWithFormat:@"%@/%@-%@-%i.tiff", [rootURL path], @"capture", d, global_gn++];
    }
    
    NSURL *u = [NSURL fileURLWithPath:filename];
    return u;
}


-(void)capture:(uint32_t)count ofDuration:(float)duration toURL:(NSURL*)url {
    abortCaptureSequence = NO;
    NSLog(@"Starting capture of %i exposures of %f seconds to %@", count, duration, url);
    for(uint32_t currentFrame=1; (currentFrame<=count) && !abortCaptureSequence && !shutdownQueue; currentFrame++ ) {
        NSURL *fileURL = [self createURL:url];
        NSLog(@"Capturing %i of %i to %@", currentFrame,count, fileURL);
        [self takeExposure:duration updateGUI:(currentFrame%10)?NO:YES saveToFile:fileURL];
    }
    NSLog(@"Complete capture");
}

-(void) takeExposure:(float)duration updateGUI:(BOOL)updateGUI saveToFile:(NSURL*)fileUrl {
    try {
        ImageFrame  *image = [[ImageFrame alloc]initWithSize:0 at:nil];
        
        
        // if you want to play around with subframes this is how it's done..
//        int midX = [_imager xPixels]/2;
//        int midY = [_imager yPixels]/2;
//        int topX = [_imager xPixels];
//        int topY = [_imager yPixels];
        
        // quartersized frames - origin (x,y), size (x,y)
//        NSRect  blquadIC24Frame = NSMakeRect(0.0,0.0, midX,midY); // bottom left
//        NSRect  tlquadIC24Frame = NSMakeRect(0.0,midY, midX, midY); // top left
//        NSRect  trquadIC24Frame = NSMakeRect(midX,midY, midX,midY); // top right
//        NSRect  brquadIC24Frame = NSMakeRect(midX,0.0, midX,midY); // bottom right
//        NSRect  centreIC24Frame = NSMakeRect(midX/2,midY/2, midX,midY); // quad in the centre
//        
//        NSRect  starFrame = NSMakeRect(537,537, 50,50);
        
       // [image setCameraFrame:starFrame];
        image.duration=duration;
        image.preview = [_previewMode boolValue];
        image.binning = NSMakePoint([_binX floatValue], [_binY floatValue]);
        image.subsampled = [_enableSubsample boolValue];
        
        [_imager setExtension:kArtemisExtensionKeyPrecharge value:(([self.enablePrecharge intValue]!=0)?@"YES":@"NO") ];
        [_imager setExtension:kArtemisExtensionKeySubsample value:(([self.enableSubsample intValue]!=0)?@"YES":@"NO") ];
        [_imager setExtension:kArtemisExtensionKeyTitanAutoBlackLevel value:(([self.enableTitanAutoBlackLevel intValue]!=0)?@"YES":@"NO") ];
        
        
        [_imager prepare:image];
        [image selfAllocate]; // note the image buffer is released when ImageFrame is destroyed
        [_imager snapShot:image blocking:YES];
        
        // leave the rest to the parallel post processing queue
        [_imager postProcess:image];
        
        ATKImageSub *sub = [[ATKImageSub alloc]initWithImageFrame:image]; // group required buffers so ARC does not free..
        
        imageB = imageA;
        subB = subA;
        imageA = image;
        subA = sub;
        
        // replace current image but using GUI main thread
         if(updateGUI) {
            [_liveView performSelectorOnMainThread:@selector(replaceLiveImage:) withObject:sub waitUntilDone:YES];
            
            //[_liveviewImage explicitReleaseSelfAllocated];
            _liveviewImage = sub;
            
        }
        
        if(fileUrl!=nil) {
            // we're not going to wait for the file to write.. not on the image queue..
            NSData *tiff = [sub TIFFRepresentation];
            [tiff writeToURL:fileUrl atomically:YES];
        }
        

    } catch( ArtemisDeviceDisconnectedException &d ) {
        NSLog(@"caught device disconnected exception");
    } catch( ... ) {
        NSLog(@" default");
    }
    
}

-(void) abort {
    abortPreview = YES;
    abortCaptureSequence = YES;
    [_imager abortCapture]; // abort capture
}


-(void)setTargetTemperature:(NSNumber *)temperature {
    setPointTemperature = [temperature floatValue];
}

-(NSNumber*)targetTemperature {
    return [NSNumber numberWithFloat:setPointTemperature];
}

-(void)setTemperature:(NSNumber *)temperature {
    self.temperature = temperature;
}

-(NSNumber*)temperature {
    return [NSNumber numberWithFloat:currentTemperature];
}


-(void) startWarmupCycle {
    dispatch_async(commandQueue, ^{
        try {
            [[_imager cooling] enableCooling:NO];  // disable will start warmup cycle automatically.
        } catch( ArtemisDeviceDisconnectedException d) {
        } catch(...) {
        }
    });
}

-(void)pollTemperatureUpdate:(NSTimer*)timer {

    if(pollSet)
        return;
    pollSet = true; // note as we're serialised by the dispatch queue and we don't need strict atomic
    
    dispatch_async(commandQueue, ^{
        try {
            if( oldSetPointTemperature!=setPointTemperature) {
                [[_imager cooling]setTemperature:setPointTemperature];
                oldSetPointTemperature = setPointTemperature;
            }
            currentTemperature=[[_imager cooling]temperature];
                        
            [_liveView performSelector:@selector(setTemperature:)
                              onThread:[NSThread mainThread]
                            withObject:[NSNumber numberWithFloat:currentTemperature]
                         waitUntilDone:NO];
            
            pollSet=false;
            
            [self setCurrentState:[_imager state]];
            
            
            
        } catch(ArtemisDeviceDisconnectedException *e) {
        } catch(...) {
        }
    });
}

-(NSString*)identity {
    return [_imager uniqueIdentity];
}

-(NSImage*)icon {
    NSLog(@"Using icon %@", _imager.model);
    return [NSImage imageNamed:_imager.model];
}

-(NSString*)model {
    return [_imager model];
}

-(void)guide:(long)direction {
    [[_imager guidePort]guide:direction];
}

@end
