//
//  ATKCameraLiveViewController.m
//  ExampleApplication
//
//  Created by Nick Kitchener on 24/01/2013.
//  Copyright (c) 2013 atik. All rights reserved.
//

#import "ATKCameraLiveViewController.h"

@interface ATKCameraLiveViewController ()

@end

@implementation ATKCameraLiveViewController

#if __MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_8
@synthesize statusView=_statusView;
@synthesize imageView=_imageView;
@synthesize cameraController=_cameraController;
@synthesize identity=_identity;
@synthesize exposure=_exposure;
@synthesize numberOfExposures=_numberOfExposures;
@synthesize targetTemperature=_targetTemperature;
@synthesize temperature=_temperature;
@synthesize storagePath=_storagePath;
@synthesize currentState=_currentState;
#endif


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Initialization code here.
        _exposure = [NSNumber numberWithFloat:0.0];
    }
    return self;
}

-(void)replaceLiveImage:(NSImage*)newImage {
    if([NSThread currentThread]!=[NSThread mainThread]) {
        NSLog(@"Thread invasion into GUI in %s", __FUNCTION__);
    }
    
    CGImageRef cgImage = [newImage CGImageForProposedRect:NULL
                                                  context:[NSGraphicsContext currentContext]
                                                    hints:[[NSDictionary alloc]init]];
    [_imageView setEditable:YES];
    [_imageView setDoubleClickOpensImageEditPanel:YES];
    [_imageView setImage:cgImage imageProperties:[[NSDictionary alloc]init]];
    [_imageView needsDisplay];
}

-(void)setDeviceController:(ATKCameraController*)ctrl {
    if([NSThread currentThread]!=[NSThread mainThread]) {
        NSLog(@"Thread invasion into GUI in %s", __FUNCTION__);
    }
    _cameraController = ctrl;
    ctrl.liveView=self;
}

-(NSString*)identity {
    return _cameraController.identity;
}

-(void)setExposure:(NSNumber *)exposure {
    [_cameraController setDuration:exposure];
    _exposure = exposure;
}

-(NSNumber*)exposure {
    _exposure = [_cameraController duration];
    return _exposure;
}

-(void)setTargetTemperature:(NSNumber *)targetTemperature {
    [_cameraController setTargetTemperature:targetTemperature];
}

-(NSNumber*)targetTemperature {
    return [_cameraController targetTemperature];
}

-(NSNumber*)temperature {
    return [_cameraController temperature]; // exec here
}

-(void)setPreviewMode:(NSNumber*)mode {
    [_cameraController setPreviewMode:mode];
}

-(NSNumber*)previewMode {
    return [_cameraController previewMode];
}

-(void)setXBin:(NSNumber*)binX {
    [_cameraController setBinX:binX];
}

-(NSNumber*)xBin {
    return [_cameraController binX];
}

-(void)setYBin:(NSNumber*)binY {
    [_cameraController setBinY:binY];
}

-(NSNumber*)yBin {
    return [_cameraController binY];
}

-(NSNumber*)enableSubsample {
    return [_cameraController enableSubsample];
}

-(void)setEnableSubsample:(NSNumber*)enable {
    [_cameraController setEnableSubsample:enable];
}

-(NSNumber*)enablePrecharge {
    return [_cameraController enablePrecharge];
}

-(void)setEnablePrecharge:(NSNumber*)enable {
    [_cameraController setEnablePrecharge:enable];
}

-(NSNumber*)enableTitanAutoBlackLevel {
    return [_cameraController enableTitanAutoBlackLevel];
}

-(void)setEnableTitanAutoBlackLevel:(NSNumber*)enable {
    [_cameraController setEnableTitanAutoBlackLevel:enable];
}

-(NSNumber*)prechargeValue {
    return [_cameraController prechargeValue];
}

-(void)setPrechargeValue:(NSNumber*)value {
    return [_cameraController setPrechargeValue:value];
}

-(void)startPreviewMode {
    NSLog(@"Starting preview mode on %@", _cameraController.identity);
    [_statusView setHidden:NO];
    [_cameraController startPreviewMode];
}

-(void)startCapture {
    NSLog(@"Starting capture mode on %@", _cameraController.identity);
    [_statusView setHidden:NO];
    uint32_t count = (uint32_t) self.numberOfExposures;
    float duration = [self.exposure floatValue];
    [_cameraController startCapture:count ofDuration:duration toURL:self.storagePath];
}

-(void)startWarmUp {
    [_cameraController startWarmupCycle];
}

-(void)abort {
    NSLog(@"abort");
    [_cameraController abort];
}

-(void)guide:(long) direction {
    [_cameraController guide:direction];
}

-(NSString*)currentState {
    return [_cameraController currentState];
}

-(void)setStateObserver:(NSObject<StateObserver>*)observer {
    [_cameraController setStateObserver:observer];
}


@end
