//
//  ATKDeviceControllerManager.m
//  ExampleApplication
//
//  Created by Nick Kitchener on 25/01/2013.
//  Copyright (c) 2013 atik. All rights reserved.
//

#import "ATKDeviceControllerManager.h"
#import "ATKCameraController.h"
#import "ATKFilterWheelController.h"

@implementation ATKDeviceControllerManager

-(id)init {
    self = [super init];
    if(self) {
        _identityToDeviceController = [[NSMutableDictionary alloc]initWithCapacity:10];
        _managedObjectArray = [[NSMutableArray alloc] initWithCapacity:10];
        _arrayControllerDelegate=nil;
    }
    return self;
}

@end

@implementation ATKDeviceControllerManager (ATIKOSXDriverNotificationDelegate)


-(id<ServiceInterface>) claimDevice:(NSString*)identity {
    id device = [_modernDrivers claimService:identity ];
    if( device == nil ) {
        device = [_legacyDrivers claimService:identity];
    }
    return device;
}

// return the device controller for a given idenitity
// If the device controller doesn't exist then the device is claimed and
// the controller is created.
-(id<ATKControllerManagement>)deviceController:(NSString*)deviceIdentity {
    
    id deviceController =[_identityToDeviceController objectForKey:deviceIdentity];
    if( deviceController==nil) {
        // no controller - claim and make
        id <ServiceInterface> device = [self claimDevice:deviceIdentity];
        if( device != nil ) {
            if( [device.model compare:@"ATIK EFW2"] != NSOrderedSame ) {
                deviceController = [[ATKCameraController alloc]initWithImager:(id<Imager100>)device];
            } else {
                deviceController = [[ATKFilterWheelController alloc]initWithFilterWheel:(id<FilterWheel100>)device];
            }
        } else
            return nil; // no device available!
    }
    return deviceController;
}


// needs to be thread safe.
-(void)deviceConnected:(NSString*)identity {
    @synchronized(self) {
        id<ATKControllerManagement> dc = [self deviceController:identity];
        if(dc!=nil) {
            // claim the device from the driver - pass this through
            //[dc setIdentity:identity];
            
            [_identityToDeviceController setObject:dc forKey:[dc identity]];
            if(_arrayControllerDelegate==nil) {
                [_managedObjectArray addObject:dc];
            } else { // array controllers are GUI so mainthread only..
                [_arrayControllerDelegate performSelectorOnMainThread:@selector(addObject:) withObject:dc waitUntilDone:NO];
            }
        }
    }
}


-(void)deviceDisconnected:(NSString*)identity {
    @synchronized(self) {
        NSObject <ATKControllerManagement> *tc=[_identityToDeviceController objectForKey:identity];
        if(tc!=nil) {
            [_identityToDeviceController removeObjectForKey:[tc identity]];
            // driver diconnected - already removed from the driver by the driver management.
            [tc shutdown]; // shutdown so we halt the temperature update 
            
            if(_arrayControllerDelegate==nil) {
                [_managedObjectArray removeObject:tc];
            } else { // array controllers are GUI so mainthread only..
                [_arrayControllerDelegate performSelectorOnMainThread:@selector(removeObject:) withObject:tc waitUntilDone:NO];
            }

        }
    }
}

@end
