//
//  ATKFilterWheelView.m
//  ExampleApplication
//
//  Created by Nick Kitchener on 08/02/2013.
//  Copyright (c) 2013 ATIK. All rights reserved.
//

#import "ATKFilterWheelView.h"

@implementation ATKFilterWheelView

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        currentImage = [self visualise];
    }
    
    return self;
}

-(id)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if(self) {
        currentImage = [self visualise];
    }
    return self;
}

- (void)drawRect:(NSRect)dirtyRect
{
    [currentImage drawAtPoint:NSMakePoint(0.0, 0.0)
                     fromRect:NSMakeRect(0.0, 0.0, currentImage.size.width, currentImage.size.height)
                    operation:NSCompositeCopy
                     fraction:1.0];
}

-(void)drawCircleAt:(NSPoint)centre withRadius:(float)radius {
    NSBezierPath *circlePath = [NSBezierPath bezierPath];
    NSPoint c=NSMakePoint(centre.x+radius, centre.y);
    [circlePath moveToPoint:c];
    [[NSColor cyanColor]setStroke];
    [circlePath setLineWidth:20];
    [circlePath appendBezierPathWithArcWithCenter:centre radius:radius startAngle:0.0 endAngle:360.0];
    [circlePath stroke];
}

// draws a line between circles given the same centre points
-(void)drawCircleLineFrom:(NSPoint)start to:(NSPoint)end withRadius:(float)radius {
    
    // do some maths to correct the length & starting point of line given the circle is 2 wide & radius wide
    double b=end.x-start.x;
    double a=end.y-start.y;
    double A=atan2(a, b);
    
    double r=radius+1;
    
    NSPoint s = NSMakePoint(start.x+r*cos(A), start.y+r*sin(A));
    NSPoint e = NSMakePoint(end.x-r*cos(A), end.y-r*sin(A));
    
    NSBezierPath *linePath = [NSBezierPath bezierPath];
    [linePath moveToPoint:s];
    [[NSColor cyanColor]setStroke];
    [linePath setLineWidth:2.0];
    [linePath lineToPoint:e];
    [linePath stroke];
}

-(NSImage*)visualise {
    NSRect offscreenRect = NSMakeRect(0.0, 0.0, self.frame.size.width , self.frame.size.height);
    NSBitmapImageRep* offscreenRep = nil;
    
    offscreenRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:nil
                                                           pixelsWide:offscreenRect.size.width
                                                           pixelsHigh:offscreenRect.size.height
                                                        bitsPerSample:8
                                                      samplesPerPixel:3
                                                             hasAlpha:NO
                                                             isPlanar:NO
                                                       colorSpaceName:NSCalibratedRGBColorSpace
                                                         bitmapFormat:0
                                                          bytesPerRow:(4 * offscreenRect.size.width)
                                                         bitsPerPixel:4*8];
    
    [NSGraphicsContext saveGraphicsState];
    [NSGraphicsContext setCurrentContext:[NSGraphicsContext graphicsContextWithBitmapImageRep:offscreenRep]];
    
//    NSColor *cSet = [NSColor colorWithCalibratedRed:1.0 green:1.0 blue:1.0 alpha:1.0];
//    NSColor *cClear = [NSColor colorWithCalibratedRed:1.0 green:1.0 blue:1.0 alpha:0.0];
    
    // where we have coverage we put in the source
    //    [_image drawAtPoint:NSMakePoint(0.0, 0.0)
    //            fromRect:offscreenRect
    //           operation:NSCompositeSourceAtop
    //            fraction:0.8];
    
    // POI points
    
    float maxRadius = (self.frame.size.width>self.frame.size.height)?self.frame.size.width:self.frame.size.height;
    
    [self drawCircleAt:NSMakePoint(self.frame.size.width/2.0, self.frame.size.height/2.0) withRadius:maxRadius/2.0];
    
    
    //    [self drawCircleAt:NSMakePoint(_image.size.width/2.0, _image.size.height/2.0) withRadius:10.0];
    //    [self drawCircleAt:NSMakePoint(_image.size.width/2.0+80, _image.size.height/2.0+90) withRadius:10.0];
    //
    //    [self drawCircleLineFrom:NSMakePoint(_image.size.width/2.0, _image.size.height/2.0)
    //                          to:NSMakePoint(_image.size.width/2.0+80, _image.size.height/2.0+90)
    //                  withRadius:10.0];
    
    
    
    [NSGraphicsContext restoreGraphicsState];
    
    NSImage *image = [[NSImage alloc]init];
    [image addRepresentation:offscreenRep];
    return image;
}

//-(NSString*)identity;
//-(NSString*)model;
//-(NSImage*)icon;
//
//-(void)shutdown; // shutdown the controller and release


@end
