//
//  ATKLiveViewHIDController.h
//  ExampleApplication
//
//  Created by Nick Kitchener on 01/02/2013.
//  Copyright (c) 2013 atik. All rights reserved.
//

// Live view manual HID window by allowing setting of:
//  * exposure duration
//  * number of exposures
//  * immediate capture (continuous)
//  * abort

#import <Cocoa/Cocoa.h>
#import "ATKLiveViewHIDDelegate.h"

@interface ATKLiveViewHIDController : NSWindowController <NSPathControlDelegate, StateObserver> {
    NSNumber    *_exposure;
    NSUInteger  _numberOfExposures;
    NSNumber    *_targetTemperature;
    NSTimer     *temperatureUpdateTimer;
    
#if __MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_8
#warning creating variables
    NSString    *_title;
    NSNumber    *_exposure;
    NSUInteger  _numberOfExposures;
    NSNumber    *_targetTemperature;
    NSNumber    *_temperature;
    NSNumber    *_previewModeActive;
    NSNumber    *_previewModeActive;
    NSNumber    *_xBin;
    NSNumber    *_yBin;
    NSNumber    *_enableSubsample;
    NSNumber    *_enablePrecharge;
    NSNumber    *_enableTitanAutoBlackLevel;
    NSNumber    *_prechargeValue;
    NSString    *_currentState;
    IBOutlet NSTextField    *_stateBox;
    NSView      *_guidePadView;
    NSURL       *_storagePath;
    IBOutlet NSPathControl *_pathControl;
    NSObject<ATKLiveViewHIDDelegate> *_hidDelegate;
    IBOutlet NSTextField *_tempLabel;
#endif    
    
}

@property(readwrite, assign, nonatomic) NSString    *title;
@property(readwrite, retain, nonatomic) NSNumber    *exposure;
@property(readwrite, assign, nonatomic) NSUInteger  numberOfExposures;

@property(readwrite, assign, nonatomic) NSNumber    *targetTemperature;
@property(readonly, assign, nonatomic) NSNumber    *temperature;

@property(readwrite, assign, nonatomic) NSNumber    *previewModeActive;
@property(readwrite, assign, nonatomic) NSNumber    *liveStackActive;

@property(readwrite, assign, nonatomic) NSNumber    *xBin;
@property(readwrite, assign, nonatomic) NSNumber    *yBin;

@property(readwrite, assign, nonatomic) NSNumber    *enableSubsample;
@property(readwrite, assign, nonatomic) NSNumber    *enablePrecharge;
@property(readwrite, assign, nonatomic) NSNumber    *enableTitanAutoBlackLevel;
@property(readwrite, assign, nonatomic) NSNumber    *prechargeValue;

@property(readwrite, assign, nonatomic) NSString    *currentState;
@property(readwrite, assign, nonatomic) IBOutlet NSTextField    *stateBox;


@property(readwrite, assign, nonatomic) NSView      *guidePadView;

@property(readwrite, assign, nonatomic) NSURL       *storagePath;

@property(readwrite, assign, nonatomic) IBOutlet NSPathControl *pathControl;

@property(readwrite, assign, nonatomic) NSObject<ATKLiveViewHIDDelegate> *hidDelegate;

@property(readwrite, assign, nonatomic) IBOutlet NSTextField *tempLabel;

-(void) temperatureUpdate:(NSTimer*)theTimer ;

-(IBAction)previewLiveViewButtonPushed:(id)sender;
-(IBAction)captureSequenceButtonPushed:(id)sender;
-(IBAction)warmUpCycle:(id)sender;

-(IBAction)GuidePortNorth:(id)sender;
-(IBAction)GuidePortSouth:(id)sender;
-(IBAction)GuidePortEast:(id)sender;
-(IBAction)GuidePortWest:(id)sender;

-(IBAction)abortButton:(id)sender; // abort current operation.

@end
