//
//  ATKLiveViewHIDController.m
//  ExampleApplication
//
//  Created by Nick Kitchener on 01/02/2013.
//  Copyright (c) 2013 atik. All rights reserved.
//

#import "ATKLiveViewHIDController.h"

@interface ATKLiveViewHIDController ()

@end

@implementation ATKLiveViewHIDController

#if __MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_8
#warning synthesizing
@synthesize title=_title;
@synthesize exposure=_exposure;
@synthesize numberOfExposures=_numberOfExposures;
@synthesize targetTemperature=_targetTemperature;
@synthesize temperature=_temperature;
@synthesize previewModeActive=_previewModeActive;
@synthesize xBin=_xBin;
@synthesize yBin=_yBin;
@synthesize enableSubsample=_enableSubsample;
@synthesize enablePrecharge=_enablePrecharge;
@synthesize enableTitanAutoBlackLevel=_enableTitanAutoBlackLevel;
@synthesize prechargeValue=_prechargeValue;
@synthesize currentState=_currentState;
@synthesize stateBox=_stateBox;
@synthesize guidePadView=_guidePadView;
@synthesize storagePath=_storagePath;
@synthesize pathControl=_pathControl;
@synthesize hidDelegate=_hidDelegate;
@synthesize tempLabel=_tempLabel;
#endif

- (id)initWithWindow:(NSWindow *)window
{
    self = [super initWithWindow:window];
    if (self) {
        // Initialization code here.
        _currentState = @"initialising..";
        _title = @"Live View Control - <unselected>";
        _exposure = [[NSNumber alloc]initWithFloat:1.0f];
        _numberOfExposures = 1;
        temperatureUpdateTimer = [NSTimer timerWithTimeInterval:2.0 target:self selector:@selector(temperatureUpdate:) userInfo:nil repeats:YES];
        [[NSRunLoop currentRunLoop]addTimer:temperatureUpdateTimer forMode:NSDefaultRunLoopMode];
        [_pathControl setURL:_storagePath];

    }
    
    return self;
}

-(void)shutdown {
    [temperatureUpdateTimer invalidate];
    temperatureUpdateTimer = nil;
}

- (void)windowDidLoad
{
    [super windowDidLoad];
    // Implement this method to handle any initialization after your window controller's window has been loaded from its nib file.
}

-(void)setHidDelegate:(NSObject<ATKLiveViewHIDDelegate> *)delegate {
    // new delegate
    @synchronized(self ) {
        _hidDelegate = delegate;
        if(_hidDelegate!=nil) {
            [_hidDelegate setStateObserver:self];
            [self setTitle:[NSString stringWithFormat:@"Live View Control - %@", _hidDelegate.identity]];
            
            [self setExposure:[_hidDelegate exposure]];
            [self setNumberOfExposures:[_hidDelegate numberOfExposures]];
            [self setTargetTemperature:[_hidDelegate targetTemperature]];
            
            if( (_hidDelegate.storagePath!=nil) || ([_hidDelegate.storagePath.path length]==0))
                [self.pathControl setURL:_hidDelegate.storagePath];
            else {
                [self.pathControl setURL:self.storagePath];
                [self.hidDelegate setStoragePath:self.storagePath];
            }
        }
    }
}

-(void)setExposure:(NSNumber *)exp {
    _exposure = exp;
    [_hidDelegate setExposure:_exposure];
}

-(void)setNumberOfExposures:(NSUInteger)exposures {
    _numberOfExposures = exposures;
    [_hidDelegate setNumberOfExposures:_numberOfExposures];
}

-(void)setTargetTemperature:(NSNumber *)valueInDegC {
    _targetTemperature = valueInDegC;
    [_hidDelegate setTargetTemperature:_targetTemperature];
}

-(NSNumber*)exposure {
    _exposure = [_hidDelegate exposure];
    return _exposure;
}

-(NSUInteger)numberOfExposures {
    _numberOfExposures = [_hidDelegate numberOfExposures];
    return _numberOfExposures;
}

-(NSNumber*)targetTemperature {
    _targetTemperature = [_hidDelegate targetTemperature];
    return _targetTemperature;
}

-(void)setPreviewModeActive:(NSNumber *)active {
    [_hidDelegate setPreviewMode:active];
}

-(NSNumber*)previewModeActive {
    return [_hidDelegate previewMode];
}

-(void)setXBin:(NSNumber *)binning {
    [_hidDelegate setXBin:binning];
}

-(NSNumber*)xBin {
    return [_hidDelegate xBin];
}

-(void)setYBin:(NSNumber *)binning {
    [_hidDelegate setYBin:binning];
}

-(NSNumber*)yBin {
    return [_hidDelegate yBin];
}

// need to check if the device is ok..
-(void) temperatureUpdate:(NSTimer*)theTimer {
    @synchronized(self ) {
        if( _hidDelegate !=nil ) {
            [self.tempLabel setStringValue:[[_hidDelegate temperature] stringValue]];
            [self.tempLabel setNeedsDisplay];
        }
    }
}

-(NSNumber*)temperature {
    return [_hidDelegate temperature];
}

-(NSNumber*)enableSubsample {
    return [_hidDelegate enableSubsample];
}

-(void)setEnableSubsample:(NSNumber*)enable {
    [_hidDelegate setEnableSubsample:enable];
}

-(NSNumber*)enablePrecharge {
    return [_hidDelegate enablePrecharge];
}

-(void)setEnablePrecharge:(NSNumber*)enable {
    [_hidDelegate setEnablePrecharge:enable];
}

-(NSNumber*)enableTitanAutoBlackLevel {
    return [_hidDelegate enableTitanAutoBlackLevel];
}

-(void)setEnableTitanAutoBlackLevel:(NSNumber*)enable {
    [_hidDelegate setEnableTitanAutoBlackLevel:enable];
}

-(NSNumber*)prechargeValue {
    return [_hidDelegate prechargeValue];
}

-(void)setPrechargeValue:(NSNumber*)value {
    [_hidDelegate setPrechargeValue:value];
}

-(NSString*)currentState {
    return [_hidDelegate currentState];
}

-(void)setState:(NSString *)newState {
//    [self setCurrentState:newState];
    [self.stateBox setStringValue:newState];
    [self.stateBox setNeedsDisplay];
}

-(IBAction)warmUpCycle:(id)sender {
    [_hidDelegate startWarmUp];
    [self setTargetTemperature:[NSNumber numberWithFloat:20.0]]; // set to 20 degC just to make sure it's not going to yoyo.
}

-(IBAction)previewLiveViewButtonPushed:(id)sender {
    [_hidDelegate startPreviewMode];
}

-(IBAction)captureSequenceButtonPushed:(id)sender {
    [_hidDelegate setStoragePath:_pathControl.URL];
    [_hidDelegate startCapture];
}

-(IBAction)abortButton:(id)sender {
    [_hidDelegate abort];
}

-(IBAction)GuidePortNorth:(id)sender {
    [_hidDelegate guide:1];
}

-(IBAction)GuidePortSouth:(id)sender {
    [_hidDelegate guide:2];
}

-(IBAction)GuidePortEast:(id)sender {
    [_hidDelegate guide:3];
}

-(IBAction)GuidePortWest:(id)sender {
    [_hidDelegate guide:4];
}

@end
