//
//  ATKMainWindowController.m
//  ExampleApplication
//
//  Created by Nick Kitchener on 24/01/2013.
//  Copyright (c) 2013 atik. All rights reserved.
//

#import "ATKMainWindowController.h"

#import "ATKTestController.h"
#import "ATKCameraLiveViewController.h"
#import "ATKFilterWheelLiveViewController.h"
#include <ATIKOSXDrivers/Logger.h>

@interface ATKMainWindowController ()

@end

@implementation ATKMainWindowController

#if __MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_8
@synthesize liveViewHIDController=_liveViewHIDController;
@synthesize selectionIndexSet=_selectionIndexSet;
@synthesize liveViewParentView=_liveViewParentView;
@synthesize  currentLiveViewController=_currentLiveViewController;
@synthesize  serialToChipId=_serialToChipId;
@synthesize  arrayController=_arrayController;
@synthesize deviceControllerManager=_deviceControllerManager;
@synthesize collectionView=_collectionView;
@synthesize debugCheckBox=_debugCheckBox;
#endif

- (id)initWithWindow:(NSWindow *)window {
    self = [super initWithWindow:window];
    if (self) {
        viewControllers = [[NSMutableDictionary alloc]initWithCapacity:10];
        _selectionIndexSet = [[NSIndexSet alloc]init];
        _serialToChipId = [[NSMutableDictionary alloc]initWithCapacity:4];
        _debugCheckBox = [NSNumber numberWithBool:NO];
    }
    
    return self;
}

-(id) initWithPath:(NSString *)newPath {
    return [super initWithWindowNibName:@"ATKMainWindowController"];
}

- (void)windowDidLoad {
    [super windowDidLoad];
    
    // Implement this method to handle any initialization after your window controller's window has been loaded from its nib file.
    [_deviceControllerManager setArrayControllerDelegate:self.arrayController];
}

-(IBAction)selectCollectionViewItem:(id)sender {
    if([NSThread currentThread]!=[NSThread mainThread]) {
        NSLog(@"Thread invasion into GUI in %s", __FUNCTION__);
    }
    
    id obr = [_arrayController selection];
    if( (obr == NSMultipleValuesMarker)
       || (obr==NSNoSelectionMarker)
       || (obr==NSNotApplicableMarker) ) {
        NSLog(@"oops");
    } else {
        BOOL newCam = NO;
        BOOL cam=NO;
          
        NSString *identity = [obr valueForKey:@"identity"];
        
        NSLog(@"Changing live view to %@", identity);
        
        id vc=[viewControllers objectForKey:identity];
        id<ATKControllerManagement> controller = [_deviceControllerManager.identityToDeviceController objectForKey:identity];
                
        if( vc==nil ) {
            // new object - check using the model to determine
            if( [controller.model compare:@"ATIK EFW2"]== NSOrderedSame ) {
                vc=[[ATKFilterWheelLiveViewController alloc]initWithNibName:@"ATKFilterWheelLiveViewController" bundle:nil];
                [vc setFilterWheelController:(ATKFilterWheelController*) controller ];
                ((ATKFilterWheelLiveViewController*)vc).filterWheelController.liveView=vc;
                [viewControllers setObject:vc forKey:identity];
            } else {
                vc=[[ATKCameraLiveViewController alloc]initWithNibName:@"ATKCameraLiveViewController" bundle:nil];
                [vc setCameraController:(ATKCameraController*) controller ];
                ((ATKCameraLiveViewController*)vc).cameraController.liveView=vc;
                [viewControllers setObject:vc forKey:identity];
                
                newCam = YES;
                cam=YES;
            }
            
        } else {
            if( [controller.model compare:@"ATIK EFW2"]!=NSOrderedSame)
                cam=YES;
        }
        
        
        // activate if an imager, otherwise deactivate
        if( cam==YES) {
            [self activateLiveViewHID];
        } else {
            [self deactivateLiveViewHID];
        }
            
        [self willChangeValueForKey:@"viewController"];
        if(cam==YES)
            _currentLiveViewController = (ATKCameraLiveViewController*) vc;
        else
            _currentLiveViewController = (ATKFilterWheelLiveViewController*) vc;
        
        NSView *v = [_currentLiveViewController view];
        
        [_liveViewParentView setSubviews:[NSArray arrayWithObject:v]];
        
        // make sure we automatically resize the controller's view to the current window size
        [[_currentLiveViewController view] setFrame: [_liveViewParentView bounds]];
        
        [v setTranslatesAutoresizingMaskIntoConstraints:NO];

        
        [_liveViewParentView addConstraints:
         [NSLayoutConstraint constraintsWithVisualFormat:@"H:|[customView]|"
                                                 options:0
                                                 metrics:nil
                                                   views:[NSDictionary dictionaryWithObject:v forKey:@"customView"]]];
        
        [_liveViewParentView addConstraints:
         [NSLayoutConstraint constraintsWithVisualFormat:@"V:|[customView]|"
                                                 options:0
                                                 metrics:nil
                                                   views:[NSDictionary dictionaryWithObject:v forKey:@"customView"]]];
        
        
        if(cam)
            [_liveViewHIDController setHidDelegate:vc];
        
        if(newCam) {
            NSLog(@"Testing cam with testExposure");
//            dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
//            dispatch_async(queue, ^{
                [((ATKCameraLiveViewController*)vc).cameraController testExposure];
//            });
        }
        
        [self didChangeValueForKey:@"viewController"];	// this will trigger the NSTextField's value binding to change
    }
}

-(void)activateLiveViewHID {
    if([NSThread currentThread]!=[NSThread mainThread]) {
        NSLog(@"Thread invasion into GUI in %s", __FUNCTION__);
    }
    
    if( _liveViewHIDController==nil ) {
        _liveViewHIDController = [[ATKLiveViewHIDController alloc] initWithWindowNibName:@"ATKLiveViewHIDController"];
    }
    [_liveViewHIDController showWindow:self];
}

-(void)deactivateLiveViewHID {
    if(_liveViewHIDController!=nil) {
        [_liveViewHIDController close];
        [_liveViewHIDController setHidDelegate:nil];
        _liveViewHIDController = nil;
    }
}

-(void)setDebugCheckBox:(NSNumber *)debugCheckBox {
    _debugCheckBox = debugCheckBox;
    Logger::setDebugLogging([debugCheckBox boolValue]);
}

@end
